#!/usr/bin/python
# -*- coding: utf-8 -*-

__author__ = 'lovro'
__doc__ = '''
ModelMMORPG - Login & Data collector
Distributed under GPL.
Implemented by Lovro Predovan, GUI and smaller edits by Markus Schatten
Copyright 2015. AI Lab FOI
'''
__version__ = '1.2'

import os
import sys
import stat

try:
    sys.path.append(os.path.join(os.path.dirname(os.path.realpath(__file__)), os.pardir))
except NameError:  # We are the main py2exe script, not a module
    sys.path.append(os.path.join(os.path.dirname(os.path.realpath(sys.argv[0])), os.pardir))

import traceback
import platform
from sys import platform as _platform
from os import listdir
from os.path import isfile, join

import subprocess
import psutil
import time
import getpass
import tempfile
import shlex

from setuptools.command import easy_install

# parsers
from parsers.battle import Battle
from parsers.general import General
from parsers.debug import Debug
from parsers.party import Party
from parsers.whisper import Whisper
from parsers.trades import Trade
from parsers.config import Config

import requests
import json

import Tkinter as tk
import ttk
import tkMessageBox as tkmb
from PIL import ImageTk, Image

import webbrowser

from urllib2 import urlopen
from urllib import urlretrieve
import io

username = ''

BASE_URL = 'http://161.53.120.2:5000/mw/api/v1/'

class Client:
	# constants
	GAME_SERVER_IP = "dragon.foi.hr" #"161.53.120.2"  # or address 
	BASE_URL = 'http://161.53.120.2:5000/mw/api/v1/'
	ARGS = ""
	SPACE = "THIS_IS_A_SPACE"

	PATH_TO_LOGS_FOLDER = ""
	PATH_TO_CONFIG_FOLDER = ""

	os = ""
	process_name = ""
	process = None

	# user_character
	character = ''

	def __init__(self):
		pass

	def __del__(self):
		self.send_chat_logs_to_server()
		try:
			self.process.kill()
			print "Killing Mana plus...\n"
			print "Thank you for contribution. Goodbye \n"

		except BaseException:
			print "THANK YOU FOR YOUR CONTRIBUTION!"


	def check_os(self):
		"""
		Method that checks users OS and sets params
		"""
		if _platform == "linux" or _platform == "linux2":
			# UBUNTU 14.04
			self.os = "linux"
			self.PATH_TO_LOGS_FOLDER = os.path.join(os.path.sep, "home", self.get_computer_user(), ".local", "share",
													"mana", "logs",
													self.GAME_SERVER_IP, self.get_current_month(),
													self.get_current_day())
			self.PATH_TO_CONFIG_FOLDER = os.path.join( os.path.sep, "home", self.get_computer_user(), ".config", "mana", 
													"mana", self.GAME_SERVER_IP  )

		elif _platform == "darwin": # TODO: add config folder for mac and win
			self.os = "mac"
			self.PATH_TO_LOGS_FOLDER = os.path.join(os.path.sep, "Users", self.get_computer_user(),
													"Library", "Application Support",
													"ManaPlus", "logs",
													self.GAME_SERVER_IP, self.get_current_month(),
													self.get_current_day())


		elif _platform == "win32" or _platform == "cygwin":
			# Windows 7
			self.os = "win"
			self.PATH_TO_LOGS_FOLDER = os.path.join("C:\Users",
													self.get_computer_user(), "AppData"
													, "Local", "Mana", "logs",
													self.GAME_SERVER_IP, self.get_current_month(),
													self.get_current_day())
			self.PATH_TO_CONFIG_FOLDER = os.path.join("C:\Users",
													self.get_computer_user(), "AppData"
													, "Roaming", "mana", "mana",
													self.GAME_SERVER_IP )

	def start_mana_plus(self):
		"""
		Method that starst mana plus client

		:return:
		"""
		if (self.os == "mac"):
			try:
				bashCommand = "open -a ManaPlus" 
				self.start_process( bashCommand, 'ManaPlus' )
			except OSError as e:
				if e.errno == os.errno.ENOENT:
					return False

		if (self.os == "linux"):
			try:
				bashCommand = "manaplus"
				self.start_process( bashCommand, 'manaplus' )
			except OSError as e:
				if e.errno == os.errno.ENOENT:
					return False

		if (self.os == "win"):
			try:
				bashCommand = "C:\Program Files (x86)\Mana\manaplus.exe"
				self.start_process( bashCommand, 'manaplus.exe' )
			except OSError as e:
				bashCommand = "C:\Program Files\Mana\manaplus.exe"
				self.start_process( bashCommand, 'manaplus.exe' )
			except Exception, e:
				print str( e )
				if e.errno == os.errno.ENOENT:
					return False
	 
	def start_process( self, bashCommand, proc_name ):
		self.process_name = proc_name
		bashCommand += self.ARGS
		cmd = [ i.replace( self.SPACE, ' ' ) for i in bashCommand.split() ]
		self.process = subprocess.Popen( cmd, stdout=subprocess.PIPE )
		if self.os == "mac":
			output = process.communicate()[0]
		self.start_process_listener()
		
	def start_process_listener(self):
		print "Dragon starting..."
		start_time = time.time()
		while 1:
			running = False
			time.sleep(1)

			for proc in psutil.process_iter():
				try:
					if self.process.pid == proc.pid:
						running = True
						status = proc.as_dict( attrs=[ "status" ] )[ "status" ]
						if status == "zombie":
							running = False 

				except (KeyboardInterrupt, SystemExit):
					return 0

				except Exception, e:					
					print str( e )
					continue

			if time.time() - start_time > 4:  # no of seconds
				if self.character != '':
					self.get_chat_logs()
				self.check_username()
				start_time = time.time()

			if not running:
				return 0
	
	def check_username( self ):
		''' Get list of characters for a given user '''
		config_file = 'config.xml'
		path = os.path.join( self.PATH_TO_CONFIG_FOLDER, config_file )
		fh = open( path )
		for line in fh.readlines():
			if 'lastCharacter' in line:
				self.character = line.split( '"' )[ -2 ]

	def get_chat_logs(self):
		"""
		Method that checks if log directory exists and parses directory structure
		After that based on params it makes call to relevant methods for parsing logs

		"""
		battle_file = '#Battle.log'
		debug_file = '#Debug.log'
		general_file = '#General.log'
		party_file = '#Party.log'
		trade_file = '#Trade.log'
		gm_file = "#GM.log"
		config_file = 'config.xml'
		
		if os.path.exists( self.PATH_TO_CONFIG_FOLDER ):
			try:
				config = Config()
				config.get_config_log_data( os.path.join( self.PATH_TO_CONFIG_FOLDER, config_file ), self.character )
			except:
				pass
		if not os.path.exists(self.PATH_TO_LOGS_FOLDER):
			print "Path does not exist  yet, please write something to chat to create folders..."
			return 0
		files = [f for f in listdir(self.PATH_TO_LOGS_FOLDER) if isfile(join(self.PATH_TO_LOGS_FOLDER, f))]
		for log_file in files:
			if log_file == battle_file:
				try:
					battle = Battle()
					battle.get_battle_log_data(os.path.join(self.PATH_TO_LOGS_FOLDER, battle_file), self.character)
					#print 1
				except:
					continue

			elif log_file == debug_file:
				try:
					debug = Debug()
					debug.get_debug_log_data(os.path.join(self.PATH_TO_LOGS_FOLDER, debug_file), self.character)
					#print 2
				except:
					continue

			elif log_file == general_file:
				try:
					general = General()
					general.get_general_log_data(os.path.join(self.PATH_TO_LOGS_FOLDER, general_file),
												 self.character)
					#print 3
				except:
					continue

			elif log_file == party_file:
				try:
					party = Party()
					party.get_party_log_data(os.path.join(self.PATH_TO_LOGS_FOLDER, party_file), self.character)
					#print 4
				except:
					continue

			elif log_file == trade_file:
				try:
					trade = Trade()
					trade.get_trades_log_data(os.path.join(self.PATH_TO_LOGS_FOLDER, trade_file), self.character)
					#print 5
				except:
					continue

			elif log_file[0] == ".":
				continue

			# if log is nothing frome above then(probably) we have whisper log
			elif log_file == gm_file:
				continue
			else:
				try:
					whisper = Whisper()
					whisper.get_whisper_log_data(os.path.join(self.PATH_TO_LOGS_FOLDER, log_file), log_file,
												 self.character)
					#print 6
				except:
					continue

	def get_character( self, user, character, callback ):
		"""
		Method that gets user char and checks if it exists on server
		returns char name
		"""
		if character == 'new':
			'''The user just registered, start manaplus'''
			callback()
			self.start_mana_plus()
			return True
		
		
		chosen = False

		char = {"character": character }
		character = {"character": character, "user": user }
		payload = json.dumps(character)
		headers = {'content-type': 'application/json'}
		try:
			r = requests.post( self.BASE_URL + "user", data=payload, headers=headers)
		except:
			tkmb.showerror( "Error connecting to server", "Unable to reach ModelMMORPG server. Check your connection!" )
			return None
		try:
			data = r.json()
		except:
			data = r.json
		try:
			if data['status'] == "200":
				self.character = char[ "character" ]
				callback()
				self.start_mana_plus()
				return True

		except Exception, e:
			print str( e )
		
		return False


	def send_chat_logs_to_server(self):
		if self.os == "mac":
			print "Sending logs to server"


	def get_computer_user(self):
		"""
		Method that provides current osx user so we could read filesystem tree to logs
		:return:
		"""
		return getpass.getuser()


	def get_current_month(self):
		"""
		Method that formats year and month so we could read current logs dir
		:return:
		"""
		return time.strftime("%Y-%m")


	def get_current_day(self):
		"""
		Method that provides current day in month
		:return:
		"""
		return time.strftime("%d")


class Login( tk.Frame ):
	BANNER = "http://modelmmorpg.foi.hr/img/logo.png"
	ICON = "http://modelmmorpg.foi.hr/img/icon.png"
	BASE_URL = 'http://161.53.120.2:5000/mw/api/v1/'
	
	def __init__( self, client, master=None ):
		''' Initialize (client is an instance of class Client) '''
		self.update = True
		
		if self.update:
			master.withdraw()
			a = AutoUpdate( tk.Toplevel(), self )
			self.update = False
			master.deiconify()
		
		tk.Frame.__init__( self, master, background="white" )
		self.pack()
		
		self.username = tk.StringVar()
		self.password = tk.StringVar()
		self.character = tk.StringVar()
		
		self.create_ui()
		self.client = client
		self.client.check_os()
		
		master.title( "The Quest for the Dragon Egg - Login" )
		master[ "bg" ] = "white"
		master.resizable( width=0, height=0 )
		self.img_icon = self.load_img( self.ICON )
		master.tk.call( 'wm', 'iconphoto', master._w, self.img_icon )
		
		master.bind( "<Return>", lambda e: self.login( e ) )
		
		self.master = master
		
	def load_img( self, url ):
		''' Load UI graphics from web '''
		try:
			raw_data = urlopen( url ).read()
		except:
			tkmb.showerror( "Error connecting to server", "Unable to reach ModelMMORPG server. Check your connection!" )
			return None
		im = Image.open( io.BytesIO( raw_data ) )
		return ImageTk.PhotoImage( image=im )
	
	def create_ui( self ):	
		''' Create user interface '''
		self.img_logo = self.load_img( self.BANNER )
		self.lbl_banner = tk.Label( self, image=self.img_logo, background="white" )
		self.lbl_banner.grid( { "row":0, "columnspan":4, "sticky":"WENS" } )
		
		self.lbl_welcome = tk.Label( self, text="Login to The Quest for the Dragon Egg - The Mana World", fg="green4", cursor="hand2", background="white" )
		self.lbl_welcome.grid( { "row":1, "columnspan":4, "sticky":"WENS", "pady":5 } )
		self.lbl_welcome.bind( "<Button-1>", lambda e: self.blog( e ) )
		
		self.frm_entries = tk.Frame( self, background="white" )
		
		self.lbl_username = tk.Label( self.frm_entries, text="Username:", background="white" )
		self.lbl_username.grid( { "row":0, "column":0, "pady":5, "sticky":"E" } )
		
		self.ent_username = tk.Entry( self.frm_entries, textvariable=self.username, width=15 )
		self.ent_username.grid( { "row":0, "column":1, "pady":5, "sticky":"W" } )
		self.ent_username.focus_set()
		
		self.lbl_password = tk.Label( self.frm_entries, text="Password:", background="white" )
		self.lbl_password.grid( { "row":1, "column":0, "pady":5, "sticky":"E" } )
		
		self.ent_password = tk.Entry( self.frm_entries, textvariable=self.password, show="*", width=15 )
		self.ent_password.grid( { "row":1, "column":1, "pady":5, "sticky":"W" } )
		
		self.lbl_character = tk.Label( self.frm_entries, text="Character:", background="white" )
		self.lbl_character.grid( { "row":2, "column":0, "pady":5, "sticky":"E" } )
		
		self.frm_characters = tk.Frame( self.frm_entries, background="white" )
		
		self.opm_character = tk.OptionMenu( self.frm_characters, self.character, "Enter credentials ..." )
		self.opm_character.pack( side="left" )
		
		self.btn_load = tk.Button( self.frm_characters, text="Load", command=self.load_characters )
		self.btn_load.pack( side="right" )
		
		self.btn_create = tk.Button( self.frm_characters, text="Create", command=self.create_and_load )
		self.btn_create.pack( side="right" )
		
		self.frm_characters.grid( { "row":2, "column":1, "sticky":"WENS", "pady":5 } )
		
		self.frm_entries.grid( { "row":2, "column":1, "columnspan":3, "sticky":"WENS", "pady":5 } )
		
		self.frm_buttons = tk.Frame( self, background="white" )
		
		self.btn_cancel = tk.Button( self.frm_buttons, text="Cancel", command=self.quit )
		self.btn_cancel.grid( { "row":0, "column":0 } )
		
		self.btn_login = tk.Button( self.frm_buttons, text="Login", command=self.login )
		self.btn_login.grid( { "row":0, "column":1 } )
		
		self.frm_buttons.grid( { "row":3, "column":2, "columnspan":2, "sticky":"WENS", "pady":5 } )
		
		self.lbl_register = tk.Label( self, text="Not registered yet? Click here!", fg="green4", cursor="hand2", background="white" )
		self.lbl_register.grid( { "row":4, "columnspan":4, "sticky":"WENS", "pady":5 } )
		self.lbl_register.bind( "<Button-1>", self.register )
		
	
	def register( self, event ):
		''' start registration form '''
		self.register = Register( tk.Toplevel(), self )
	
	def blog( self, event ):
		''' Open the web portal '''
		webbrowser.open_new( r"http://dragon.foi.hr" )
		print "Open portal!"
	
	def load_characters( self, event=None ):
		''' Load characters into option widget '''
		chars = self.get_char_list()
		self.opm_character[ "menu" ].delete( 0, "end" )
		self.character.set( "" )
		if chars != 'no such user': 
			if len( chars ) == 0:
				self.opm_character[ "menu" ].add_command( label="Please create a character", command=tk._setit( self.character, "Please create a character" ) )
			else:
				self.character.set( chars[ 0 ] )
				for c in chars:
					self.opm_character[ "menu" ].add_command( label=c, command=tk._setit( self.character, c ) )
		else:
			tkmb.showwarning( "Invalid credentials", "Please enter a valid username and password!" )
	
	def get_char_list( self ):
		''' Get list of characters for a given user '''
		char = { "user": self.username.get() }
		payload = json.dumps( char )
		headers = { 'content-type': 'application/json' }
		try:
			r = requests.post( self.BASE_URL + "list_chars", data=payload, headers=headers )
		except:
			tkmb.showerror( "Error connecting to server", "Unable to reach ModelMMORPG server. Check your connection!" )
			return None
		
		try:
			result = r.json()
		except:
			result = r.json
		try:
			if result.has_key( "error" ):
				return result[ "error" ]
			else:
				return result[ "characters" ]
		except:
			return []
	
	def create_and_load( self, event=None ):
		self.login( create_char=True )
	
	def login( self, event=None, create_char=False ):
		''' Login to ManaWorld server '''
		if create_char:
			char = 'new'
		else:
			char = self.character.get()
		if char == '' or char == 'Enter credentials ...':
			tkmb.showwarning( "Invalid character", "Please select a valid character!" )
			return None
		user = self.username.get()
		if create_char:
			self.client.ARGS = " -U %s -P %s -s dragon.foi.hr" % ( user.replace( ' ', self.client.SPACE ), self.password.get().replace( ' ', self.client.SPACE ) )
		else:
			self.client.ARGS = " -c %s -U %s -P %s -s dragon.foi.hr" % ( char.replace( ' ', self.client.SPACE ), user.replace( ' ', self.client.SPACE ), self.password.get().replace( ' ', self.client.SPACE ) )
		if self.check_credentials():
			if self.client.get_character( user, char, self.master.destroy ):
				pass
			else:
				tkmb.showwarning( "Invalid character", "Please select a valid character!" )
		else:
			tkmb.showerror( "Invalid credentials", "Invalid username, password or character!" )
	
	def check_credentials( self ):
		''' Check is username and password are ok '''
		credentials = { "username": self.username.get(), "password": self.password.get() }
		payload = json.dumps( credentials )
		headers = { 'content-type': 'application/json' }
		
		try:
			r = requests.post( self.BASE_URL + "check_password", data=payload, headers=headers )
		except:
			tkmb.showerror( "Error connecting to server", "Unable to reach ModelMMORPG server. Check your connection!" )
			return None

		try:
			data = r.json()
		except:
			data = r.json
			
		try:
			return int( data[ "status" ] ) == 200
		except:
			return False

class Register( tk.Frame ):
	def __init__( self, master, login ):
		''' Initialize registration form (login is instance of Login) '''
		tk.Frame.__init__( self, master, background="white" )
		self.pack()
		
		self.username = tk.StringVar()
		self.password1 = tk.StringVar()
		self.password2 = tk.StringVar()
		self.sex = tk.StringVar()
		self.email = tk.StringVar()
		self.terms = tk.IntVar()
		
		self.login = login
		
		self.create_ui()
		
		master.title( "The Quest for the Dragon Egg - Register" )
				
		login.master.withdraw()
				
		self.master = master
		
		master.protocol( "WM_DELETE_WINDOW", self.on_delete )
		master.bind( "<Destroy>", self.on_delete )
		
		self.just_registered = False
	
	def on_delete( self, event=None ):
		''' Make login window visible again if this one is destroyed '''
		try:
			self.master.destroy()
		except:
			pass
		if not self.just_registered:
			self.login.master.update()
			self.login.master.deiconify()
		else:
			self.quit()
	
	def create_ui( self ):
		''' Create user interface '''
		self.img_logo = self.login.load_img( self.login.BANNER )
		self.lbl_banner = tk.Label( self, image=self.img_logo, background="white" )
		self.lbl_banner.grid( { "row":0, "columnspan":4, "sticky":"WENS" } )
		
		self.lbl_welcome = tk.Label( self, text="Register for The Quest for the Dragon Egg - The Mana World", fg="green4", cursor="hand2", background="white" )
		self.lbl_welcome.grid( { "row":1, "columnspan":4, "sticky":"WENS", "pady":5 } )
		self.lbl_welcome.bind( "<Button-1>", lambda e: self.login.blog( e ) )
	
		self.frm_entries = tk.Frame( self, background="white" )
		
		self.lbl_username = tk.Label( self.frm_entries, text="Username:", background="white" )
		self.lbl_username.grid( { "row":0, "column":0, "pady":5, "sticky":"E" } )
		
		self.ent_username = tk.Entry( self.frm_entries, textvariable=self.username, width=15 )
		self.ent_username.grid( { "row":0, "column":1, "pady":5, "sticky":"W" } )
		self.ent_username.focus_set()
		
		self.lbl_password1 = tk.Label( self.frm_entries, text="Password:", background="white" )
		self.lbl_password1.grid( { "row":1, "column":0, "pady":5, "sticky":"E" } )
		
		self.ent_password1 = tk.Entry( self.frm_entries, textvariable=self.password1, show="*", width=15 )
		self.ent_password1.grid( { "row":1, "column":1, "pady":5, "sticky":"W" } )
		
		self.lbl_password2 = tk.Label( self.frm_entries, text="Repeat:", background="white" )
		self.lbl_password2.grid( { "row":2, "column":0, "pady":5, "sticky":"E" } )
		
		self.ent_password2 = tk.Entry( self.frm_entries, textvariable=self.password2, show="*", width=15 )
		self.ent_password2.grid( { "row":2, "column":1, "pady":5, "sticky":"W" } )
		
		self.lbl_email = tk.Label( self.frm_entries, text="E-mail:", background="white" )
		self.lbl_email.grid( { "row":3, "column":0, "pady":5, "sticky":"E" } )
		
		self.ent_email = tk.Entry( self.frm_entries, textvariable=self.email, width=15 )
		self.ent_email.grid( { "row":3, "column":1, "pady":5, "sticky":"W" } )
		
		self.lbl_sex = tk.Label( self.frm_entries, text="Sex:", background="white" )
		self.lbl_sex.grid( { "row":4, "column":0, "pady":5, "sticky":"E" } )
		
		self.opm_sex = tk.OptionMenu( self.frm_entries, self.sex, "Female", "Male" )
		self.opm_sex.grid( { "row":4, "column":1, "pady":5, "sticky":"W" } )
				
		self.lbl_terms = tk.Label( self.frm_entries, text="Terms &\nConditions", fg="green4", cursor="hand2", background="white"  )
		self.lbl_terms.grid( { "row":5, "column":0, "pady":5, "sticky":"E" } )
		self.lbl_terms.bind( "<Button-1>", lambda e: webbrowser.open_new( r"http://modelmmorpg.foi.hr/tmw/terms.html" ) )
		
		self.opm_terms = tk.Checkbutton( self.frm_entries, variable=self.terms, text="I agree", background="white", highlightbackground="white", selectcolor="white" )
		self.opm_terms.grid( { "row":5, "column":1, "pady":5, "sticky":"W" } )
		
		self.frm_entries.grid( { "row":2, "column":1, "columnspan":3, "sticky":"WENS", "pady":5 } )
		
		self.frm_buttons = tk.Frame( self, background="white" )
		
		self.btn_cancel = tk.Button( self.frm_buttons, text="Cancel", command=self.on_delete )
		self.btn_cancel.grid( { "row":0, "column":0 } )
		
		self.btn_register = tk.Button( self.frm_buttons, text="Register", command=self.register )
		self.btn_register.grid( { "row":0, "column":1 } )
		
		self.frm_buttons.grid( { "row":3, "column":2, "columnspan":2, "sticky":"WENS", "pady":5 } )
	
	def register( self, event=None ):
		''' Register the user (check if all entries are valid before that) '''
		user = self.username.get()
		pass1 = self.password1.get()
		pass2 = self.password2.get()
		email = self.email.get()
		sex = self.sex.get()
		terms = self.terms.get()
		
		if len( user ) < 4:
			tkmb.showwarning( "Invalid username", "Please enter a username of at least 4 characters!" )
			return
		if len( user ) > 23:
			tkmb.showwarning( "Invalid username", "Please enter a username of at most 23 characters!" )
			return
		if pass1 != pass2:
			tkmb.showwarning( "Passwords do not match", "Please enter two equal passwords!" )
			return
		if len( pass1 ) < 4:
			tkmb.showwarning( "Invalid password", "Please enter a password of at least 4 characters!" )
			return
		if len( pass1 ) > 23:
			tkmb.showwarning( "Invalid password", "Please enter a password of at most 23 characters!" )
			return
		if '@' not in email:
			tkmb.showwarning( "Invalid email", "Please enter a valid email address!" )
			return
		if sex == '':
			tkmb.showwarning( "Invalid sex", "Please choose a gender!" )
			return
		if not( terms ):
			tkmb.showwarning( "Terms & Conditions", "Please agree to terms and conditions!" )
			return
		
		if sex == 'Male':
			sex = 'M'
		else:
			sex = 'F'
		
		global username
		username = user
		
		credentials = { "username": user, "password": pass1, "sex":sex, "email":email }
		payload = json.dumps( credentials )
		headers = { 'content-type': 'application/json' }
		try:
			r = requests.post( self.login.BASE_URL + "register", data=payload, headers=headers )
		except:
			tkmb.showerror( "Error connecting to server", "Unable to reach ModelMMORPG server. Check your connection!" )
			return None
		
		try:
			data = r.json()
		except:
			data = r.json
		
		if int( data[ "status" ] ) != 200:
			tkmb.showwarning( "Registration error", "There has been an error during registration!\nMaybe the username is already taken." )
		else:
			self.just_registered = True
			tkmb.showinfo( "Registration successful", "You have been registered succesfully!" )
			self.login.client.ARGS = " -U %s -P %s -s dragon.foi.hr" % ( user.replace( ' ', self.login.client.SPACE ), pass1.replace( ' ', self.login.client.SPACE ) )
			self.destroy()
			self.login.client.get_character( user, 'new', self.login.destroy )

class AutoUpdate( tk.Frame ):
	'''Automated updater'''
	BASE_URL = 'http://161.53.120.2:5000/mw/api/v1/'
	def __init__( self, master, ultimatemaster ):
		
		url = self.get_download_url()
		master.withdraw()
		if url:
			res = tkmb.askquestion( "Update", "There is a newer version of the Dragon egg quest client available. Upgrade now?" )
			if res == "yes":
			
				master.deiconify()
				tk.Frame.__init__( self, master, background="white" )
				self.pack()
				
				self.img_icon = ultimatemaster.load_img( ultimatemaster.ICON )
				master.tk.call( 'wm', 'iconphoto', master._w, self.img_icon )
			
				master.title( "Downloading new version" )
				self.lbl_text = tk.Label( self, text="Downloading new version of the Dragon egg quest. Please wait...", background="white"  )
				self.lbl_text.grid( { "row":1, "column":1, "pady":5, "sticky":"W" } )
				self.progress = ttk.Progressbar( self, orient="horizontal", length=200, mode="determinate" )
				self.progress.grid( { "row":2, "column":1, "pady":5 } )
				self.bytes = 0
				self.maxbytes = 0
				try:
					fl = self.download( url )
					self.update( fl )
				except:
					# user clicked the x button
					pass
				sys.exit()
		
	def download( self, url ):
		folder = tempfile.mkdtemp()
		fl = os.path.join( folder, url.split( '/' )[ -1 ] )
		file_name = url.split('/')[ -1 ]
		u = urlopen( url )
		f = open( fl, 'wb')
		meta = u.info()
		file_size = int( meta.getheaders( "Content-Length" )[ 0 ] )
		
		file_size_dl = 0
		block_sz = 8192
		
		self.progress["value"] = 0
		self.maxbytes = file_size
		self.progress["maximum"] = 100
		
		while True:
			buffer = u.read( block_sz )
			if not buffer:
				break
			file_size_dl += len( buffer )
			f.write( buffer )
			p = 100 * float( file_size_dl ) / file_size
			self.lbl_text[ "text" ] = "Downloading the newest version of the Dragon egg quest client. Please wait!\nReceiving: %.1f / %.1f MiB [%.1f]%% " % ( float( file_size_dl ) / 1024 / 1024, float( file_size  ) / 1024 / 1024, p )
			self.progress[ "value" ] = p
			self.progress.update()

		f.close()
		return fl
	
	def update( self, fl ):
		if _platform == "linux" or _platform == "linux2":
			subprocess.call(shlex.split('gksudo easy_install ' + fl))
		else:
			p = subprocess.Popen( fl, close_fds=True, shell=True )
	
	def get_download_url( self ):
		req = { "version":__version__, "os":_platform }
		payload = json.dumps( req )
		headers = { 'content-type': 'application/json' }
		try:
			r = requests.post( self.BASE_URL + "update", data=payload, headers=headers )
		except:
			tkmb.showerror( "Error connecting to server", "Unable to reach ModelMMORPG server. Check your connection!" )
			return None
		try:
			data = r.json()
		except:
			data = r.json
		
		try:
			if data:
				if data[ "update" ] == "yes":
					return data[ "download" ]
		except:
			pass

def report( error, stack ):
	os = platform.system() + ' ' + platform.release()
	req = { "version":__version__, "os":os, "error":error, "stack":stack }
	payload = json.dumps( req )
	headers = { 'content-type': 'application/json' }
	try:
		r = requests.post( BASE_URL + "error", data=payload, headers=headers )
	except:
		tkmb.showerror( "Error connecting to server", "Unable to reach ModelMMORPG server. Check your connection!" )
		return
	
	tkmb.showwarning( "Whoops ... ", "Dragon Egg client has crashed! We are sorry for the inconvenience. The developers have been notified, and will hopefully find a solution soon." )

def main():
    try:
		root = tk.Tk()
		client = Client()
		app = Login( client, master=root )
		app.mainloop()
		
		try:
			root.destroy()
		except:
			pass
    except Exception, e:
		report( str( e ), traceback.format_exc() )


if __name__ == '__main__':
	main()
